@@ -21,6 +21,9 @@
 #include "utils/palloc.h"
 
 
+#define MAX_PUT_COPY_DATA_BUFFER_SIZE (8 * 1024 * 1024)
+
+
 /* GUC, determining whether statements sent to remote nodes are logged */
 bool LogRemoteCommands = false;
 
@@ -179,7 +182,8 @@ NonblockingForgetResults(MultiConnection *connection)
 		}
 
 		result = PQgetResult(pgConn);
-		if (PQresultStatus(result) == PGRES_COPY_IN)
+		if (PQresultStatus(result) == PGRES_COPY_IN ||
+			PQresultStatus(result) == PGRES_COPY_OUT)
 		{
 			/* in copy, can't reliably recover without blocking */
 			return false;
@@ -580,11 +584,20 @@ PutRemoteCopyData(MultiConnection *connection, const char *buffer, int nbytes)
 	 * until the socket is writable to prevent the internal libpq buffers
 	 * from growing excessively.
 	 *
-	 * In the future, we could reduce the frequency of these pushbacks to
-	 * achieve higher throughput.
+	 * We currently allow the internal buffer to grow to 8MB before
+	 * providing back pressure based on experimentation that showed
+	 * throughput get worse at 4MB and lower due to the number of CPU
+	 * cycles spent in networking system calls.
 	 */
 
-	return FinishConnectionIO(connection, allowInterrupts);
+	connection->copyBytesWrittenSinceLastFlush += nbytes;
+	if (connection->copyBytesWrittenSinceLastFlush > MAX_PUT_COPY_DATA_BUFFER_SIZE)
+	{
+		connection->copyBytesWrittenSinceLastFlush = 0;
+		return FinishConnectionIO(connection, allowInterrupts);
+	}
+
+	return true;
 }
 
 
@@ -616,6 +629,8 @@ PutRemoteCopyEnd(MultiConnection *connection, const char *errormsg)
 
 	/* see PutRemoteCopyData() */
 
+	connection->copyBytesWrittenSinceLastFlush = 0;
+
 	return FinishConnectionIO(connection, allowInterrupts);
 }
 
@@ -766,6 +781,7 @@ WaitForAllConnections(List *connectionList, bool raiseInterrupts)
 
 		while (pendingConnectionsStartIndex < totalConnectionCount)
 		{
+			bool cancellationReceived = false;
 			int eventIndex = 0;
 			int eventCount = 0;
 			long timeout = -1;
@@ -822,9 +838,13 @@ WaitForAllConnections(List *connectionList, bool raiseInterrupts)
 					if (InterruptHoldoffCount > 0 && (QueryCancelPending ||
 													  ProcDiePending))
 					{
-						/* return immediately in case of cancellation */
-						FreeWaitEventSet(waitEventSet);
-						return;
+						/*
+						 * Break out of event loop immediately in case of cancellation.
+						 * We cannot use "return" here inside a PG_TRY() block since
+						 * then the exception stack won't be reset.
+						 */
+						cancellationReceived = true;
+						break;
 					}
 
 					continue;
@@ -890,6 +910,11 @@ WaitForAllConnections(List *connectionList, bool raiseInterrupts)
 				}
 			}
 
+			if (cancellationReceived)
+			{
+				break;
+			}
+
 			/* move non-ready connections to the back of the array */
 			for (connectionIndex = pendingConnectionsStartIndex;
 				 connectionIndex < totalConnectionCount; connectionIndex++)
