@@ -106,10 +106,10 @@ bool EnableRouterExecution = true;
 
 
 /* planner functions forward declarations */
-static MultiPlan * CreateSingleTaskRouterPlan(Query *originalQuery,
-											  Query *query,
-											  RelationRestrictionContext *
-											  restrictionContext);
+static DistributedPlan * CreateSingleTaskRouterPlan(Query *originalQuery,
+													Query *query,
+													RelationRestrictionContext *
+													restrictionContext);
 static bool MasterIrreducibleExpression(Node *expression, bool *varArgument,
 										bool *badCoalesce);
 static bool MasterIrreducibleExpressionWalker(Node *expression, WalkerState *state);
@@ -158,12 +158,10 @@ static List * MultiShardModifyTaskList(Query *originalQuery, List *relationShard
  * SELECT statement.  If planning fails either NULL is returned, or
  * ->planningError is set to a description of the failure.
  */
-MultiPlan *
+DistributedPlan *
 CreateRouterPlan(Query *originalQuery, Query *query,
 				 RelationRestrictionContext *restrictionContext)
 {
-	Assert(EnableRouterExecution);
-
 	if (MultiRouterPlannableQuery(query, restrictionContext))
 	{
 		return CreateSingleTaskRouterPlan(originalQuery, query,
@@ -183,20 +181,20 @@ CreateRouterPlan(Query *originalQuery, Query *query,
  * statement.  If planning fails ->planningError is set to a description of
  * the failure.
  */
-MultiPlan *
+DistributedPlan *
 CreateModifyPlan(Query *originalQuery, Query *query,
 				 PlannerRestrictionContext *plannerRestrictionContext)
 {
 	Job *job = NULL;
-	MultiPlan *multiPlan = CitusMakeNode(MultiPlan);
+	DistributedPlan *distributedPlan = CitusMakeNode(DistributedPlan);
 	bool multiShardQuery = false;
 
-	multiPlan->operation = query->commandType;
+	distributedPlan->operation = query->commandType;
 
-	multiPlan->planningError = ModifyQuerySupported(query, multiShardQuery);
-	if (multiPlan->planningError != NULL)
+	distributedPlan->planningError = ModifyQuerySupported(query, multiShardQuery);
+	if (distributedPlan->planningError != NULL)
 	{
-		return multiPlan;
+		return distributedPlan;
 	}
 
 	if (UpdateOrDeleteQuery(query))
@@ -204,31 +202,32 @@ CreateModifyPlan(Query *originalQuery, Query *query,
 		RelationRestrictionContext *restrictionContext =
 			plannerRestrictionContext->relationRestrictionContext;
 
-		job = RouterJob(originalQuery, restrictionContext, &multiPlan->planningError);
+		job = RouterJob(originalQuery, restrictionContext,
+						&distributedPlan->planningError);
 	}
 	else
 	{
-		job = RouterInsertJob(originalQuery, query, &multiPlan->planningError);
+		job = RouterInsertJob(originalQuery, query, &distributedPlan->planningError);
 	}
 
-	if (multiPlan->planningError != NULL)
+	if (distributedPlan->planningError != NULL)
 	{
-		return multiPlan;
+		return distributedPlan;
 	}
 
 	ereport(DEBUG2, (errmsg("Creating router plan")));
 
-	multiPlan->workerJob = job;
-	multiPlan->masterQuery = NULL;
-	multiPlan->routerExecutable = true;
-	multiPlan->hasReturning = false;
+	distributedPlan->workerJob = job;
+	distributedPlan->masterQuery = NULL;
+	distributedPlan->routerExecutable = true;
+	distributedPlan->hasReturning = false;
 
 	if (list_length(originalQuery->returningList) > 0)
 	{
-		multiPlan->hasReturning = true;
+		distributedPlan->hasReturning = true;
 	}
 
-	return multiPlan;
+	return distributedPlan;
 }
 
 
@@ -239,26 +238,26 @@ CreateModifyPlan(Query *originalQuery, Query *query,
  * are router plannable by default. If query is not router plannable then either NULL is
  * returned, or the returned plan has planningError set to a description of the problem.
  */
-static MultiPlan *
+static DistributedPlan *
 CreateSingleTaskRouterPlan(Query *originalQuery, Query *query,
 						   RelationRestrictionContext *restrictionContext)
 {
 	Job *job = NULL;
-	MultiPlan *multiPlan = CitusMakeNode(MultiPlan);
+	DistributedPlan *distributedPlan = CitusMakeNode(DistributedPlan);
 
-	multiPlan->operation = query->commandType;
+	distributedPlan->operation = query->commandType;
 
 	/* FIXME: this should probably rather be inlined into CreateRouterPlan */
-	multiPlan->planningError = ErrorIfQueryHasModifyingCTE(query);
-	if (multiPlan->planningError)
+	distributedPlan->planningError = ErrorIfQueryHasModifyingCTE(query);
+	if (distributedPlan->planningError)
 	{
-		return multiPlan;
+		return distributedPlan;
 	}
 
 	/* we cannot have multi shard update/delete query via this code path */
-	job = RouterJob(originalQuery, restrictionContext, &multiPlan->planningError);
+	job = RouterJob(originalQuery, restrictionContext, &distributedPlan->planningError);
 
-	if (multiPlan->planningError)
+	if (distributedPlan->planningError)
 	{
 		/* query cannot be handled by this planner */
 		return NULL;
@@ -266,12 +265,12 @@ CreateSingleTaskRouterPlan(Query *originalQuery, Query *query,
 
 	ereport(DEBUG2, (errmsg("Creating router plan")));
 
-	multiPlan->workerJob = job;
-	multiPlan->masterQuery = NULL;
-	multiPlan->routerExecutable = true;
-	multiPlan->hasReturning = false;
+	distributedPlan->workerJob = job;
+	distributedPlan->masterQuery = NULL;
+	distributedPlan->routerExecutable = true;
+	distributedPlan->hasReturning = false;
 
-	return multiPlan;
+	return distributedPlan;
 }
 
 
@@ -1622,12 +1621,22 @@ SelectsFromDistributedTable(List *rangeTableList)
  * If the given query is not routable, it fills planningError with the related
  * DeferredErrorMessage. The caller can check this error message to see if query
  * is routable or not.
+ *
+ * Note: If the query prunes down to 0 shards due to filters (e.g. WHERE false),
+ * or the query has only read_intermediate_result calls (no relations left after
+ * recursively planning CTEs and subqueries), then it will be assigned to an
+ * arbitrary worker node in a round-robin fashion.
+ *
+ * Relations that prune down to 0 shards are replaced by subqueries returning
+ * 0 values in UpdateRelationToShardNames.
  */
 DeferredErrorMessage *
 PlanRouterQuery(Query *originalQuery, RelationRestrictionContext *restrictionContext,
 				List **placementList, uint64 *anchorShardId, List **relationShardList,
 				bool replacePrunedQueryWithDummy, bool *multiShardModifyQuery)
 {
+	static uint32 zeroShardQueryRoundRobin = 0;
+
 	bool isMultiShardQuery = false;
 	List *prunedRelationShardList = NIL;
 	DeferredErrorMessage *planningError = NULL;
@@ -1729,7 +1738,10 @@ PlanRouterQuery(Query *originalQuery, RelationRestrictionContext *restrictionCon
 		List *workerNodeList = ActiveReadableNodeList();
 		if (workerNodeList != NIL)
 		{
-			WorkerNode *workerNode = (WorkerNode *) linitial(workerNodeList);
+			int workerNodeCount = list_length(workerNodeList);
+			int workerNodeIndex = zeroShardQueryRoundRobin % workerNodeCount;
+			WorkerNode *workerNode = (WorkerNode *) list_nth(workerNodeList,
+															 workerNodeIndex);
 			ShardPlacement *dummyPlacement =
 				(ShardPlacement *) CitusMakeNode(ShardPlacement);
 			dummyPlacement->nodeName = workerNode->workerName;
@@ -1737,6 +1749,8 @@ PlanRouterQuery(Query *originalQuery, RelationRestrictionContext *restrictionCon
 			dummyPlacement->groupId = workerNode->groupId;
 
 			workerList = lappend(workerList, dummyPlacement);
+
+			zeroShardQueryRoundRobin++;
 		}
 	}
 	else
@@ -2477,8 +2491,15 @@ MultiRouterPlannableQuery(Query *query, RelationRestrictionContext *restrictionC
 		{
 			/* only hash partitioned tables are supported */
 			Oid distributedTableId = rte->relid;
-			char partitionMethod = PartitionMethod(distributedTableId);
+			char partitionMethod = 0;
 
+			if (!IsDistributedTable(distributedTableId))
+			{
+				/* local tables cannot be read from workers */
+				return false;
+			}
+
+			partitionMethod = PartitionMethod(distributedTableId);
 			if (!(partitionMethod == DISTRIBUTE_BY_HASH || partitionMethod ==
 				  DISTRIBUTE_BY_NONE || partitionMethod == DISTRIBUTE_BY_RANGE))
 			{
