@@ -27,6 +27,9 @@
 #include "utils/hsearch.h"
 
 
+#define PREPARED_TRANSACTION_NAME_FORMAT "citus_%u_%u_"UINT64_FORMAT "_%u"
+
+
 static void StartRemoteTransactionSavepointBegin(MultiConnection *connection,
 												 SubTransactionId subId);
 static void FinishRemoteTransactionSavepointBegin(MultiConnection *connection,
@@ -82,7 +85,8 @@ StartRemoteTransactionBegin(struct MultiConnection *connection)
 	 */
 	distributedTransactionId = GetCurrentDistributedTransactionId();
 	appendStringInfo(beginAndSetDistributedTransactionId,
-					 "SELECT assign_distributed_transaction_id(%d, %ld, '%s');",
+					 "SELECT assign_distributed_transaction_id(%d, " UINT64_FORMAT
+					 ", '%s');",
 					 distributedTransactionId->initiatorNodeIdentifier,
 					 distributedTransactionId->transactionNumber,
 					 timestamptz_to_str(distributedTransactionId->timestamp));
@@ -1241,11 +1245,36 @@ Assign2PCIdentifier(MultiConnection *connection)
 
 	/* print all numbers as unsigned to guarantee no minus symbols appear in the name */
 	snprintf(connection->remoteTransaction.preparedName, NAMEDATALEN,
-			 "citus_%u_%u_"UINT64_FORMAT "_%u", GetLocalGroupId(), MyProcPid,
+			 PREPARED_TRANSACTION_NAME_FORMAT, GetLocalGroupId(), MyProcPid,
 			 transactionNumber, connectionNumber++);
 }
 
 
+/*
+ * ParsePreparedTransactionName parses a prepared transaction name to extract
+ * the initiator group ID, initiator process ID, distributed transaction number,
+ * and the connection number. If the transaction name does not match the expected
+ * format ParsePreparedTransactionName returns false, and true otherwise.
+ */
+bool
+ParsePreparedTransactionName(char *preparedTransactionName, int *groupId, int *procId,
+							 uint64 *transactionNumber, uint32 *connectionNumber)
+{
+	const int expectedFieldCount = 4;
+	int parsedFieldCount = 0;
+	bool nameValid = false;
+
+	parsedFieldCount = sscanf(preparedTransactionName, PREPARED_TRANSACTION_NAME_FORMAT,
+							  groupId, procId, transactionNumber, connectionNumber);
+	if (parsedFieldCount == expectedFieldCount)
+	{
+		nameValid = true;
+	}
+
+	return nameValid;
+}
+
+
 /*
  * WarnAboutLeakedPreparedTransaction issues a WARNING explaining that a
  * prepared transaction could not be committed or rolled back, and explains
