@@ -16,6 +16,7 @@
 
 #include "distributed/master_metadata_utility.h"
 #include "distributed/multi_logical_planner.h"
+#include "distributed/relation_restriction_equivalence.h"
 
 
 /* Definitions local to logical plan optimizer */
@@ -119,10 +120,9 @@ extern Oid FunctionOid(const char *schemaName, const char *functionName,
 extern List * SubqueryMultiTableList(MultiNode *multiNode);
 extern List * GroupTargetEntryList(List *groupClauseList, List *targetEntryList);
 extern bool ExtractQueryWalker(Node *node, List **queryList);
-extern bool LeafQuery(Query *queryTree);
-extern List * PartitionColumnOpExpressionList(Query *query);
-extern List * ReplaceColumnsInOpExpressionList(List *opExpressionList, Var *newColumn);
-extern bool IsPartitionColumnRecursive(Expr *columnExpression, Query *query);
+extern bool IsPartitionColumn(Expr *columnExpression, Query *query);
+extern void FindReferencedTableColumn(Expr *columnExpression, List *parentQueryList,
+									  Query *query, Oid *relationId, Var **column);
 
 
 #endif   /* MULTI_LOGICAL_OPTIMIZER_H */
