@@ -4,8 +4,8 @@
 -- Test checks whether constraints of distributed tables can be adjusted using
 -- the ALTER TABLE ... ADD CONSTRAINT ... command.
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1450000;
-ALTER SEQUENCE pg_catalog.pg_dist_placement_placementid_seq RESTART 1450000;
+SET citus.next_shard_id TO 1450000;
+SET citus.next_placement_id TO 1450000;
 
 -- Check "PRIMARY KEY CONSTRAINT"
 CREATE TABLE products (
@@ -398,10 +398,9 @@ ROLLBACK;
 
 -- There should be no constraint on master and worker(s) 
 SELECT "Constraint", "Definition" FROM table_checks WHERE relid='products'::regclass;
-
 \c - - - :worker_1_port
 
-SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450202'::regclass;
+SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450034'::regclass;
 
 \c - - - :master_port
 
@@ -418,9 +417,12 @@ SELECT "Constraint", "Definition" FROM table_checks WHERE relid='products'::regc
 
 \c - - - :worker_1_port
 
-SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450202'::regclass;
+SELECT "Constraint", "Definition" FROM table_checks WHERE relid='public.products_1450034'::regclass;
 
 \c - - - :master_port
+SET citus.next_shard_id TO 1450038;
+SET citus.next_placement_id TO 1450038;
+
 DROP TABLE products;
 
 SET citus.shard_count to 2;
