@@ -4,7 +4,7 @@
 -- tests UDFs created for citus tools
 --
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1240000;
+SET citus.next_shard_id TO 1240000;
 
 -- test with invalid port, prevent OS dependent warning from being displayed
 SET client_min_messages to ERROR;
