@@ -1,4 +1,4 @@
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1220000;
+SET citus.next_shard_id TO 1220000;
 
 -- Tests functions related to cluster membership
 
