@@ -7,7 +7,7 @@
 -- not done yet.
 
 
-ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 580000;
+SET citus.next_shard_id TO 580000;
 ALTER SEQUENCE pg_catalog.pg_dist_jobid_seq RESTART 580000;
 
 CREATE SCHEMA test;
@@ -129,6 +129,9 @@ ALTER EXTENSION citus UPDATE TO '7.1-1';
 ALTER EXTENSION citus UPDATE TO '7.1-2';
 ALTER EXTENSION citus UPDATE TO '7.1-3';
 ALTER EXTENSION citus UPDATE TO '7.1-4';
+ALTER EXTENSION citus UPDATE TO '7.2-1';
+ALTER EXTENSION citus UPDATE TO '7.2-2';
+ALTER EXTENSION citus UPDATE TO '7.2-3';
 
 -- show running version
 SHOW citus.version;
