@@ -529,7 +529,8 @@ SELECT
 	FROM
 		articles_hash_mx
  	GROUP BY
-		author_id;
+		author_id
+	ORDER BY c;
 
 -- queries inside transactions can be router plannable
 BEGIN;
