@@ -5,11 +5,9 @@
 --
 
 -- We don't need shard id sequence here, so commented out to prevent conflicts with concurrent tests
--- ALTER SEQUENCE pg_catalog.pg_dist_shardid_seq RESTART 1400000;
+-- SET citus.next_shard_id TO 1400000;
 ALTER SEQUENCE pg_catalog.pg_dist_jobid_seq RESTART 1400000;
 
-SET citus.enable_router_execution TO FALSE;
-
 CREATE TABLE user_buy_test_table(user_id int, item_id int, buy_count int);
 SELECT create_distributed_table('user_buy_test_table', 'user_id');
 INSERT INTO user_buy_test_table VALUES(1,2,1);
@@ -136,6 +134,75 @@ FROM
   ) as foo
   GROUP BY user_id ORDER BY 2 DESC LIMIT 10;
 
+-- table function can be the inner relationship in a join
+SELECT count(*) FROM
+  (SELECT random() FROM user_buy_test_table JOIN generate_series(1,10) AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1;
+
+-- table function cannot be used without subquery pushdown
+SELECT count(*) FROM user_buy_test_table JOIN generate_series(1,10) AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- table function can be the inner relationship in an outer join
+SELECT count(*) FROM
+  (SELECT random() FROM user_buy_test_table LEFT JOIN generate_series(1,10) AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1;
+
+SELECT count(*) FROM user_buy_test_table LEFT JOIN (SELECT * FROM generate_series(1,10) id) users_ref_test_table
+ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- table function cannot be the outer relationship in an outer join
+SELECT count(*) FROM
+  (SELECT random() FROM user_buy_test_table RIGHT JOIN generate_series(1,10) AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1;
+
+SELECT count(*) FROM user_buy_test_table RIGHT JOIN (SELECT * FROM generate_series(1,10) id) users_ref_test_table
+ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- volatile functions can be used as table expressions through recursive planning
+SET client_min_messages TO DEBUG;
+
+SELECT count(*) FROM
+  (SELECT random() FROM user_buy_test_table JOIN random() AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1;
+
+-- can sneak in a volatile function as a parameter
+SELECT count(*) FROM
+  (SELECT item_id FROM user_buy_test_table JOIN generate_series(random()::int,10) AS users_ref_test_table(id)
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1
+WHERE item_id = 6;
+
+-- can perform a union with table function through recursive planning
+SELECT count(*) FROM
+  (SELECT user_id FROM user_buy_test_table
+   UNION ALL
+   SELECT id FROM generate_series(1,10) AS users_ref_test_table(id)) subquery_1;
+
+RESET client_min_messages;
+
+-- subquery without FROM can be the inner relationship in a join
+SELECT count(*) FROM
+  (SELECT random() FROM user_buy_test_table JOIN (SELECT 4 AS id) users_ref_test_table
+  ON user_buy_test_table.item_id > users_ref_test_table.id) subquery_1;
+
+-- subquery without FROM triggers subquery pushdown
+SELECT count(*) FROM user_buy_test_table JOIN (SELECT 5 AS id) users_ref_test_table
+ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- subquery without FROM can be the inner relationship in an outer join
+SELECT count(*) FROM user_buy_test_table LEFT JOIN (SELECT 5 AS id) users_ref_test_table
+ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- subquery without FROM cannot be the outer relationship in an outer join
+SELECT count(*) FROM user_buy_test_table RIGHT JOIN (SELECT 5 AS id) users_ref_test_table
+ON user_buy_test_table.item_id = users_ref_test_table.id;
+
+-- can perform a union with subquery without FROM
+SELECT count(*) FROM
+  (SELECT user_id FROM user_buy_test_table
+   UNION ALL
+   SELECT id FROM (SELECT 5 AS id) users_ref_test_table) subquery_1;
+
 -- should be able to pushdown since reference table is in the
 -- inner part of the left join
 SELECT
@@ -194,14 +261,14 @@ SELECT * FROM
             FROM
               events_reference_table as "events"
             WHERE
-              event_type > 80) as "temp_data_queries"
+              event_type > 2) as "temp_data_queries"
            INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) as foo_in ON (event_val_2 = user_id)) as foo LEFT JOIN
+              user_id > 2 and value_2 = 1) as foo_in ON (event_val_2 = user_id)) as foo LEFT JOIN
            (SELECT user_id as user_user_id FROM users_table) as fooo ON (user_id = user_user_id)) as bar;
 
 -- the same query but this time reference table is in the outer part of the query
@@ -214,14 +281,14 @@ SELECT * FROM
             FROM
               events_reference_table as "events"
             WHERE
-              event_type > 80) as "temp_data_queries"
+              event_type > 2) as "temp_data_queries"
            LEFT JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) as foo_in ON (event_val_2 = user_id)) as foo LEFT JOIN
+              user_id > 2 and value_2 = 1) as foo_in ON (event_val_2 = user_id)) as foo LEFT JOIN
            (SELECT user_id as user_user_id FROM users_table) as fooo ON (user_id = user_user_id)) as bar;
 
 -- we could even suuport the following where the subquery
@@ -268,14 +335,14 @@ FROM
             FROM
               users_reference_table as "users"
             WHERE
-              user_id > 12 and user_id < 16 and value_1 > 20) user_where_1_1
+              user_id > 0 and user_id < 5 and value_1 > 1) user_where_1_1
          INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_reference_table as "users"
             WHERE
-              user_id > 12 and user_id < 16 and value_2 > 60) user_where_1_join_1
+              user_id > 0 and user_id < 5 and value_2 > 2) user_where_1_join_1
            ON ("user_where_1_1".user_id = "user_where_1_join_1".user_id))
         filter_users_1
       JOIN LATERAL
@@ -284,7 +351,7 @@ FROM
          FROM
           events_reference_table as "events"
          WHERE
-           user_id > 12 and user_id < 16 AND
+           user_id > 0 and user_id < 5 AND
            user_id = filter_users_1.user_id
          ORDER BY
           time DESC
@@ -300,7 +367,7 @@ FROM
         users_reference_table  as "users"
       WHERE
         "users"."user_id" = "some_recent_users"."user_id" AND
-        "users"."value_2" > 70
+        "users"."value_2" > 2
       LIMIT 1) "some_users_data"
     ON TRUE
    ORDER BY
@@ -329,7 +396,7 @@ SELECT
             FROM
               events_table as "events"
             WHERE
-              user_id > 10 and user_id < 40 AND event_type IN (40, 41, 42, 43, 44, 45) ) "temp_data_queries"
+              user_id > 0 and user_id < 4 AND event_type IN (4, 5) ) "temp_data_queries"
             INNER  JOIN
            (SELECT
               user_where_1_1.real_user_id
@@ -339,14 +406,14 @@ SELECT
                FROM
                 users_reference_table as "users"
                WHERE
-                 user_id > 10 and user_id < 40 and value_2 > 50 ) user_where_1_1
+                 user_id > 0 and user_id < 4 and value_2 > 3 ) user_where_1_1
             INNER JOIN
               (SELECT
                   "users"."user_id"
                FROM
                 users_reference_table as "users"
                WHERE
-                user_id > 10 and user_id < 40 and value_3 > 50 ) user_where_1_join_1
+                user_id > 0 and user_id < 4 and value_3 > 3 ) user_where_1_join_1
            ON ("user_where_1_1".real_user_id = "user_where_1_join_1".user_id)) "user_filters_1"
       ON ("temp_data_queries".user_id = "user_filters_1".real_user_id)) "eventQuery") "pushedDownQuery") "pushedDownQuery"
 GROUP BY
@@ -375,7 +442,7 @@ FROM
                  FROM
                   users_reference_table as "users"
                  WHERE
-                  user_id > 10 and user_id < 40 and value_2 > 30
+                  user_id > 1 and user_id < 4 and value_2 > 2
                 ) simple_user_where_1
             ) all_buckets_1
         ) users_in_segment_1
@@ -385,7 +452,7 @@ FROM
          FROM
           users_reference_table as "users"
          WHERE
-          user_id > 10 and user_id < 40 and value_2 > 60
+          user_id > 1 and user_id < 4 and value_2 > 3
         ) some_users_data
         ON ("users_in_segment_1".user_id = "some_users_data".user_id)
     ) segmentalias_1) "tempQuery"
@@ -406,14 +473,14 @@ FROM
          FROM
           users_reference_table as "users"
          WHERE
-          user_id > 20 and user_id < 70 and users.value_2 = 200) filter_users_1
+          user_id > 2 and user_id < 5 and users.value_2 = 3) filter_users_1
       JOIN LATERAL
         (SELECT
             user_id, value_3
          FROM
           events_reference_table as "events"
          WHERE
-           user_id > 20 and user_id < 70 AND
+           user_id > 2 and user_id < 5 AND
            ("events".user_id = "filter_users_1".user_id)
          ORDER BY
            value_3 DESC
@@ -427,7 +494,7 @@ FROM
         users_reference_table as "users"
       WHERE
         "users"."user_id" = "some_recent_users"."user_id" AND
-        users.value_2 > 200
+        users.value_2 > 3
       LIMIT 1) "some_users_data" ON true
    ORDER BY
     value_3 DESC
@@ -455,14 +522,14 @@ count(*) AS cnt, "generated_group_field"
             FROM
               events_table as "events"
             WHERE
-              user_id > 80) "temp_data_queries"
+              user_id > 4) "temp_data_queries"
            INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_reference_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) "user_filters_1"
+              user_id > 2 and value_2 = 5) "user_filters_1"
            ON ("temp_data_queries".event_user_id = "user_filters_1".user_id)) AS "multi_group_wrapper_1"
         LEFT JOIN
         (SELECT
@@ -494,14 +561,14 @@ count(*) AS cnt, "generated_group_field"
             FROM
               events_table as "events"
             WHERE
-              user_id > 80) "temp_data_queries"
+              user_id > 2) "temp_data_queries"
            INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) "user_filters_1"
+              user_id > 2 and value_2 = 5) "user_filters_1"
            ON ("temp_data_queries".event_user_id = "user_filters_1".user_id)) AS "multi_group_wrapper_1"
         RIGHT JOIN
         (SELECT
@@ -529,17 +596,17 @@ FROM (
       FROM users_table AS u,
           events_reference_table AS e
       WHERE  u.user_id > e.user_id
-      AND u.user_id >= 10
-      AND u.user_id <= 25
-      AND e.event_type IN (100, 101, 102)
+      AND u.user_id >= 1
+      AND u.user_id <= 3
+      AND e.event_type IN (1, 2)
     )
   ) t1 RIGHT JOIN (
       SELECT DISTINCT user_id,
         'Has done event'::TEXT AS hasdone_event
       FROM  events_table AS e
-      WHERE  e.user_id >= 10
-      AND e.user_id <= 25
-      AND e.event_type IN (106, 107, 108)
+      WHERE  e.user_id >= 1
+      AND e.user_id <= 3
+      AND e.event_type IN (3, 4)
   ) t2 ON (t1.user_id = t2.user_id)
   GROUP BY  t1.user_id, hasdone_event
 ) t GROUP BY user_id, hasdone_event
@@ -558,17 +625,17 @@ FROM (
       FROM users_table AS u,
           events_reference_table AS e
       WHERE  u.value_1 > e.user_id
-      AND u.user_id >= 10
-      AND u.user_id <= 25
-      AND e.event_type >= 125 AND e.event_type < 130
+      AND u.user_id >= 1
+      AND u.user_id <= 3
+      AND e.event_type >= 2 AND e.event_type < 3
     )
   ) t1 RIGHT JOIN (
       SELECT DISTINCT user_id,
         'Has done event'::TEXT AS hasdone_event
       FROM  events_table AS e
-      WHERE  e.user_id >= 10
-      AND e.user_id <= 25
-      AND e.event_type >= 130 AND e.event_type < 135
+      WHERE  e.user_id >= 1
+      AND e.user_id <= 3
+      AND e.event_type >= 3 AND e.event_type < 4
   ) t2 ON (t1.user_id = t2.user_id)
   GROUP BY  t1.user_id, hasdone_event
 ) t GROUP BY user_id, hasdone_event
@@ -594,14 +661,14 @@ count(*) AS cnt, "generated_group_field"
             FROM
               events_table as "events"
             WHERE
-              user_id > 80) "temp_data_queries"
+              user_id > 2) "temp_data_queries"
            INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_reference_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) "user_filters_1"
+              user_id > 2 and value_2 = 5) "user_filters_1"
            ON ("temp_data_queries".event_user_id < "user_filters_1".user_id)) AS "multi_group_wrapper_1"
         RIGHT JOIN
         (SELECT
@@ -627,7 +694,7 @@ FROM
             FROM
               events_reference_table as "events"
             WHERE
-              user_id > 10 and user_id < 40) "events_1"
+              user_id > 1 and user_id < 4) "events_1"
          ORDER BY
            time DESC) "recent_events_1"
       GROUP BY
@@ -640,7 +707,7 @@ FROM
       FROM
         users_table as "users"
       WHERE
-        users.value_2 > 50 and users.value_2 < 55) "some_users_data"
+        users.value_2 > 2 and users.value_2 < 4) "some_users_data"
      ON "some_users_data"."user_id" = "some_recent_users"."user_id"
 ORDER BY
   user_id
@@ -665,7 +732,7 @@ FROM
                     FROM
                       events_table as  "events"
                     WHERE
-                      event_type IN (10, 11, 12, 13, 14, 15) ) events_subquery_1)
+                      event_type IN (1, 2) ) events_subquery_1)
                  UNION
                  (SELECT
                     *
@@ -675,7 +742,7 @@ FROM
                      FROM
                         events_reference_table as "events"
                      WHERE
-                      event_type IN (15, 16, 17, 18, 19) ) events_subquery_2)
+                      event_type IN (3, 4) ) events_subquery_2)
                UNION
                  (SELECT
                     *
@@ -685,7 +752,7 @@ FROM
                      FROM
                         events_table as  "events"
                      WHERE
-                      event_type IN (20, 21, 22, 23, 24, 25) ) events_subquery_3)
+                      event_type IN (5, 6) ) events_subquery_3)
                 UNION
                   (SELECT
                       *
@@ -695,7 +762,7 @@ FROM
                      FROM
                       events_table as "events"
                      WHERE
-                      event_type IN (26, 27, 28, 29, 30, 13)) events_subquery_4)) t1
+                      event_type IN (1, 6)) events_subquery_4)) t1
          GROUP BY "t1"."user_id") AS t) "q"
 INNER JOIN
      (SELECT
@@ -703,12 +770,13 @@ INNER JOIN
       FROM
         users_table as "users"
       WHERE
-        value_1 > 50 and value_1 < 70) AS t
+        value_1 > 2 and value_1 < 4) AS t
     ON (t.user_id = q.user_id)) as final_query
 ORDER BY
-  types;
+  types
+LIMIT 5;
 
-  -- reference table exist in the subquery of union, should error out
+  -- reference table exist in the subquery of union
 SELECT ("final_query"."event_types") as types, count(*) AS sumOfEventType
 FROM
   ( SELECT
@@ -728,7 +796,7 @@ FROM
                     FROM
                       events_table as "events"
                     WHERE
-                      event_type IN (10, 11, 12, 13, 14, 15) ) events_subquery_1)
+                      event_type IN (1, 2) ) events_subquery_1)
                 UNION
                  (SELECT *
                   FROM
@@ -743,7 +811,7 @@ FROM
                                 events_reference_table as  "events", users_table as "users"
                               WHERE
                                 events.user_id = users.user_id AND
-                                event_type IN (10, 11, 12, 13, 14, 15)
+                                event_type IN (1, 2)
                                 GROUP BY   "users"."user_id"
                           ) as events_subquery_5
                      ) events_subquery_2)
@@ -755,7 +823,7 @@ FROM
                      FROM
                         events_table as  "events"
                      WHERE
-                      event_type IN (20, 21, 22, 23, 24, 25) ) events_subquery_3)
+                      event_type IN (3, 4) ) events_subquery_3)
                UNION
                  (SELECT *
                   FROM
@@ -764,7 +832,7 @@ FROM
                      FROM
                       events_table as "events"
                      WHERE
-                      event_type IN (26, 27, 28, 29, 30, 13)) events_subquery_4)
+                      event_type IN (5, 6)) events_subquery_4)
                  ) t1
          GROUP BY "t1"."user_id") AS t) "q"
 INNER JOIN
@@ -773,7 +841,7 @@ INNER JOIN
       FROM
         users_table as "users"
       WHERE
-        value_1 > 50 and value_1 < 70) AS t
+        value_1 > 2 and value_1 < 4) AS t
      ON (t.user_id = q.user_id)) as final_query
 GROUP BY
   types
@@ -798,7 +866,7 @@ FROM
                     FROM
                       events_table as  "events"
                     WHERE
-                      event_type IN (10, 11, 12, 13, 14, 15) ) events_subquery_1)
+                      event_type IN (1, 2) ) events_subquery_1)
                 UNION ALL
                  (SELECT *
                   FROM
@@ -807,7 +875,7 @@ FROM
                      FROM
                         events_table as "events"
                      WHERE
-                        event_type IN (15, 16, 17, 18, 19) ) events_subquery_2)
+                        event_type IN (3, 4) ) events_subquery_2)
                UNION ALL
                  (SELECT *
                   FROM
@@ -816,7 +884,7 @@ FROM
                      FROM
                         events_reference_table as  "events"
                      WHERE
-                        event_type IN (20, 21, 22, 23, 24, 25) ) events_subquery_3)
+                        event_type IN (5, 6) ) events_subquery_3)
                UNION ALL
                  (SELECT *
                   FROM
@@ -825,12 +893,12 @@ FROM
                      FROM
                         events_table as "events"
                      WHERE
-                        event_type IN (26, 27, 28, 29, 30, 13)) events_subquery_4)) t1
+                        event_type IN (1, 6)) events_subquery_4)) t1
          GROUP BY "t1"."user_id") AS t) "q"
 INNER JOIN
      (SELECT "users"."user_id"
       FROM users_table as "users"
-      WHERE value_1 > 50 and value_1 < 70) AS t ON (t.user_id = q.user_id)) as final_query
+      WHERE value_1 > 2 and value_1 < 4) AS t ON (t.user_id = q.user_id)) as final_query
 GROUP BY types
 ORDER BY types;
 
@@ -866,14 +934,14 @@ count(*) AS cnt, "generated_group_field"
             FROM
               events_table as "events"
             WHERE
-              user_id > 80) "temp_data_queries"
+              user_id > 2) "temp_data_queries"
            INNER JOIN
            (SELECT
               "users"."user_id"
             FROM
               users_reference_table as "users"
             WHERE
-              user_id > 80 and value_2 = 5) "user_filters_1"
+              user_id > 2 and value_2 = 5) "user_filters_1"
            ON ("temp_data_queries".event_user_id < "user_filters_1".user_id)) AS "multi_group_wrapper_1"
         RIGHT JOIN
         (SELECT
@@ -901,8 +969,8 @@ WHERE
              FROM
               events_reference_table as e2
              WHERE
-              value_2 = 15 AND
-              value_3 > 25 AND
+              value_2 = 1 AND
+              value_3 > 3 AND
               e1.value_2 > e2.value_2
             )
             AND u1.user_id > e1.user_id
@@ -913,17 +981,20 @@ LIMIT 5;
 SELECT foo.user_id FROM
 (
   SELECT m.user_id, random() FROM users_table m JOIN events_reference_table r ON int4eq(m.user_id, r.user_id)
-  WHERE event_type > 100000
+  WHERE event_type > 100
 ) as foo;
 
--- not supported since group by is on the reference table column
+-- not pushdownable since group by is on the reference table column
+-- recursively planned
 SELECT foo.user_id FROM
 (
   SELECT r.user_id, random() FROM users_table m JOIN events_reference_table r ON int4eq(m.user_id, r.user_id)
   GROUP BY r.user_id
-) as foo;
+) as foo
+ORDER BY 1 DESC;
 
--- supported since the group by contains at least one distributed table
+-- not pushdownable since the group by contains at least one distributed table
+-- recursively planned
 SELECT foo.user_id FROM
 (
   SELECT r.user_id, random() FROM users_table m JOIN events_reference_table r ON int4eq(m.user_id, r.user_id)
@@ -931,13 +1002,17 @@ SELECT foo.user_id FROM
 ) as foo
 ORDER BY 1 LIMIT 3;
 
--- not supported since distinct is on the reference table column
+-- not pushdownable since distinct is on the reference table column
+-- recursively planned
 SELECT foo.user_id FROM
 (
   SELECT DISTINCT r.user_id, random() FROM users_table m JOIN events_reference_table r ON int4eq(m.user_id, r.user_id)
-) as foo;
+) as foo
+ORDER BY 1 DESC
+LIMIT 5;
 
 -- not supported since distinct on is on the reference table column
+-- but recursively planned
 SELECT foo.user_id FROM
 (
   SELECT DISTINCT ON(r.user_id) r.user_id, random() FROM users_table m JOIN events_reference_table r ON int4eq(m.user_id, r.user_id)
@@ -950,6 +1025,221 @@ SELECT foo.user_id FROM
 ) as foo
 ORDER BY 1 LIMIT 3;
 
+-- should be able to pushdown since one of the subqueries has distinct on reference tables
+-- and there is only reference table in that subquery
+SELECT 
+  distinct_users, event_type, time
+FROM
+(SELECT user_id, time, event_type FROM events_table) as events_dist INNER JOIN
+(SELECT DISTINCT user_id as distinct_users FROM users_reference_table) users_ref ON (events_dist.user_id = users_ref.distinct_users)
+ORDER BY time DESC
+LIMIT 5
+OFFSET 0;
+
+-- the same query wuth multiple reference tables in the subquery
+SELECT 
+  distinct_users, event_type, time
+FROM
+(SELECT user_id, time, event_type FROM events_table) as events_dist INNER JOIN
+(SELECT DISTINCT users_reference_table.user_id as distinct_users FROM users_reference_table, events_reference_table 
+ WHERE events_reference_table.user_id =  users_reference_table.user_id AND events_reference_table.event_type IN (1,2,3,4)) users_ref
+ON (events_dist.user_id = users_ref.distinct_users)
+ORDER BY time DESC
+LIMIT 5
+OFFSET 0;
+
+-- similar query as the above, but with group bys
+SELECT 
+  distinct_users, event_type, time
+FROM
+(SELECT user_id, time, event_type FROM events_table) as events_dist INNER JOIN
+(SELECT user_id as distinct_users FROM users_reference_table GROUP BY distinct_users) users_ref ON (events_dist.user_id = users_ref.distinct_users)
+ORDER BY time DESC
+LIMIT 5
+OFFSET 0;
+
+-- should not push down this query since there is a distributed table (i.e., events_table)
+-- which is not in the DISTINCT clause. Recursive planning also fails since router execution
+-- is disabled
+SELECT * FROM
+(
+  SELECT DISTINCT users_reference_table.user_id FROM users_reference_table, events_table WHERE users_reference_table.user_id = events_table.value_4
+) as foo;
+
+SELECT * FROM
+(
+  SELECT users_reference_table.user_id FROM users_reference_table, events_table WHERE users_reference_table.user_id = events_table.value_4
+  GROUP BY 1
+) as foo;
+
+-- similiar to the above examples, this time there is a subquery 
+-- whose output is not in the DISTINCT clause
+SELECT * FROM
+(
+  SELECT DISTINCT users_reference_table.user_id FROM users_reference_table, (SELECT user_id, random() FROM events_table) as us_events WHERE users_reference_table.user_id = us_events.user_id
+) as foo
+ORDER BY 1;
+
+-- the following query is safe to push down since the DISTINCT clause include distribution column
+SELECT * FROM
+(
+  SELECT DISTINCT users_reference_table.user_id, us_events.user_id FROM users_reference_table, (SELECT user_id, random() FROM events_table WHERE event_type IN (2,3)) as us_events WHERE users_reference_table.user_id = us_events.user_id
+) as foo 
+ORDER BY 1 DESC 
+LIMIT 4;
+
+-- should not pushdown since there is a non partition column on the DISTINCT clause
+-- Recursive planning also fails since router execution
+-- is disabled
+SELECT * FROM
+(
+  SELECT 
+    DISTINCT users_reference_table.user_id, us_events.value_4 
+  FROM 
+    users_reference_table, 
+    (SELECT user_id, value_4, random() FROM events_table WHERE event_type IN (2,3)) as us_events 
+  WHERE 
+    users_reference_table.user_id = us_events.user_id
+) as foo 
+ORDER BY 1 DESC 
+LIMIT 4;
+
+
+
+-- test the read_intermediate_result() for GROUP BYs
+BEGIN;
+ 
+SELECT broadcast_intermediate_result('squares', 'SELECT s, s*s FROM generate_series(1,200) s');
+
+-- single appereance of read_intermediate_result
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_table 
+JOIN 
+(SELECT 
+  max(res.val) as mx 
+FROM 
+  read_intermediate_result('squares', 'binary') AS res (val int, val_square int) 
+GROUP BY res.val_square) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+-- similar to the above, with DISTINCT on intermediate result
+SELECT DISTINCT user_id
+FROM users_table
+JOIN
+  (SELECT DISTINCT res.val AS mx
+   FROM read_intermediate_result('squares', 'binary') AS res (val int, val_square int)) squares ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+-- single appereance of read_intermediate_result but inside a subquery
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_table 
+JOIN (
+  SELECT *,random() FROM (SELECT 
+    max(res.val) as mx 
+  FROM 
+      (SELECT val, val_square FROM read_intermediate_result('squares', 'binary') AS res (val int, val_square int)) res 
+  GROUP BY res.val_square) foo)
+squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+-- multiple read_intermediate_results in the same subquery is OK
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_table 
+JOIN 
+(SELECT 
+  max(res.val) as mx 
+FROM 
+  read_intermediate_result('squares', 'binary') AS res (val int, val_square int),
+  read_intermediate_result('squares', 'binary') AS res2 (val int, val_square int) 
+WHERE res.val = res2.val_square
+GROUP BY res2.val_square) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+-- mixed recurring tuples should be supported
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_table 
+JOIN 
+(SELECT 
+  max(res.val) as mx 
+FROM 
+  read_intermediate_result('squares', 'binary') AS res (val int, val_square int), 
+  generate_series(0, 10) i
+  WHERE
+  res.val = i
+  GROUP BY 
+    i) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+-- should recursively plan since
+-- there are no columns on the GROUP BY from the distributed table
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_reference_table
+JOIN 
+  (SELECT 
+    max(val_square) as mx 
+  FROM 
+    read_intermediate_result('squares', 'binary') AS res (val int, val_square int), events_table 
+  WHERE 
+    events_table.user_id = res.val GROUP BY res.val) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+ROLLBACK;
+
+-- should work since we're using an immutable function as recurring tuple
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_table 
+JOIN 
+(SELECT 
+  max(i+5)as mx 
+FROM 
+  generate_series(0, 10) as i GROUP BY i) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
+
+-- should recursively plan since we're 
+-- using an immutable function as recurring tuple
+-- along with a distributed table, where GROUP BY is 
+-- on the recurring tuple
+SELECT 
+  DISTINCT user_id 
+FROM 
+  users_reference_table
+JOIN 
+  (SELECT 
+    max(i+5)as mx 
+  FROM 
+     generate_series(0, 10) as i, events_table 
+  WHERE 
+    events_table.user_id = i GROUP BY i) squares
+ ON (mx = user_id)
+ORDER BY 1
+LIMIT 5;
+
 DROP TABLE user_buy_test_table;
 DROP TABLE users_ref_test_table;
 DROP TABLE users_return_test_table;
